@@ -29,9 +29,11 @@ WikiJS options:
   --wiki-git-no-verify              Do not verify the ssl certificate when cloning the wikijs wiki.
 
 Docker options:
-  -t --tag=<tag>                    The docker tag to use for the generated image. [default: sci-oer/custom:latest]
+  -t --tag=<tag>                    The docker tag to use for the generated image. This should exclude the registry portion. [default: sci-oer/custom:latest]
   -b --base=<base>                  The base image to use [default: marshallasch/java-resource:latest]
+  --registry=<url>                  The url of a docker registry to push the image to
   --no-pull                         Don't pull the base image first
+  --no-push                         Don't push the image to a remote registry.
   --save-tar=<file name>            The name of the tar archive to export the image as.
                                     If running in docker it will get put in the `/output` directory,
                                     if the builder is being run on the host then it will be palced in the `./output` directory.
@@ -41,9 +43,9 @@ Other interface options:
   -V --version                      Show the current version.
   -v --verbose                      Show verbose log messages.
   -d --debug                        Show debug log messages.
+  -i --interactive                  Interactivly prompt for all of the options.
 """  # noqa E501
 
-from fileinput import filename
 import docker
 import random
 import string
@@ -54,24 +56,26 @@ import shutil
 import platform
 import requests
 import json
-import stat
-from git import Repo  # noqa: I900
+import copy
+
 from docopt import docopt
-import subprocess
 import colorlog
 import logging
 from requests.adapters import HTTPAdapter
 from requests.packages.urllib3.util.retry import Retry
 import datetime
-
 import importlib.resources as pkg_resources
 
-from .__version__ import __version__  # noqa: I900
+try:
+    from git import Repo  # noqa: I900
+except:
+    sys.exit("Can not run, `git` must be installed on the system.")
 
+from .__version__ import __version__  # noqa: I900
+from .prompt import prompt, yesno, prompt_list
 
 SSH_KEY_FILE_ENV = "SSH_KEY_FILE"
 
-
 # this is the api token that has been built into the base-resource container
 API_TOKEN = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJhcGkiOjEsImdycCI6MSwiaWF0IjoxNjQyOTcyMTk5LCJleHAiOjE3Mzc2NDQ5OTksImF1ZCI6InVybjp3aWtpLmpzIiwiaXNzIjoidXJuOndpa2kuanMifQ.xkvgFfpYw2OgB0Z306YzVjOmuYzrKgt_fZLXetA0ThoAgHNH1imou2YCh-JBXSBCILbuYvfWMSwOhf5jAMKT7O1QJNMhs5W0Ls7Cj5tdlOgg-ufMZaLH8X2UQzkD-1o3Dhpv_7hs9G8xt7qlqCz_-DwroOGUGPaGW6wrtUfylUyYh86V9eJveRJqzZXiGFY3n6Z3DuzIVZtz-DoCHMaDceSG024BFOD-oexMCnAxTpk5OalEhwucaYHS2sNCLpmwiEGHSswpiaMq9-JQasVJtQ_fZ9yU_ZZLBlc0AJs1mOENDTI6OBZ3IS709byqxEwSPnWaF_Tk7fcGnCYk-3gixA"  # noqa E501
 
@@ -87,6 +91,105 @@ def _make_opts(args):
     return opts
 
 
+def ask_interactive(opts):
+    input = copy.deepcopy(opts)
+
+    # general image setup  information
+    print("## Information about the container to be built.")
+    input["tag"] = prompt(
+        "Enter the name of the image to be built, including the tag: ",
+        default=input["tag"],
+    )
+    input["base"] = prompt(
+        "Enter the base image that should be used for this course:",
+        default=input["base"],
+    )
+    input["no_pull"] = not yesno(
+        "Should a the latest base image be fetched?",
+        default="no" if input["no_pull"] else "yes",
+    )
+
+    input["no_push"] = not yesno(
+        "Do you want to push the generated image to a docker registry?",
+        default="no" if input["no_push"] else "yes",
+    )
+    if not input["no_push"]:
+        input["registry"] = prompt(
+            "Enter the registry that the generated should be pushed too (leave blank for dockerhub):",
+            default=input["registry"],
+        )
+
+    input["save_tar"] = prompt(
+        "Enter name of the tar file of the generated image to save (leave blank if you do not want to save it):",
+        default=input["save_tar"],
+    )
+
+    print("")
+    print("## General options for cloning git repositories.")
+    input["no_verify_host"] = yesno(
+        "Should the SSH host keys be verified?",
+        default="no" if input["no_verify_host"] else "yes",
+    )
+    input["keep_git"] = yesno(
+        "Should the git histories be kept after they are cloned?",
+        default="yes" if input["keep_git"] else "no",
+    )
+    input["key_file"] = prompt(
+        "Enter the path to the SSH private key used to clone the git repos if one is being used:",
+        default=input["key_file"],
+    )
+
+    print("")
+    print("## Information about the wiki to be created.")
+    input["wiki_title"] = prompt(
+        "Enter the title for the Wiki:", default=input["wiki_title"]
+    )
+    input["wiki_git_repo"] = prompt(
+        "Enter the git repository URL for the wiki:", default=input["wiki_git_repo"]
+    )
+    input["wiki_git_branch"] = prompt(
+        "Enter the branch name for the wiki repository:",
+        default=input["wiki_git_branch"],
+    )
+
+    isSSH = yesno("Do you want to clone the wiki repository using ssh?", default="yes")
+    if not isSSH:
+        input["wiki_git_user"] = prompt(
+            "Enter the git username for cloning the wiki contents:",
+            default=input["wiki_git_user"],
+        )
+        input["wiki_git_password"] = prompt(
+            "Enter the git password for cloning the wiki contents",
+            default=input["wiki_git_password"],
+        )
+
+    input["wiki_git_no_verify"] = yesno(
+        "Should the SSL certificates be verified when cloning the wiki?",
+        default="no" if input["wiki_git_no_verify"] else "yes",
+    )
+
+    print("")
+    print("## Where the rest of the content is being loaded from.")
+    input["jupyter_repo"] = prompt(
+        "Enter the git repository that holds the Jupyter Notebook files (leave blank if not being used)",
+        default=input["jupyter_repo"],
+    )
+    input["lectures_repo"] = prompt(
+        "Enter the git repository that holds the Jupyter Lecture video files (leave blank if not being used)",
+        default=input["lectures_repo"],
+    )
+    input["lectures_directory"] = prompt(
+        "Enter the directory that contains the Jupyter Lecture video files (leave blank if not being used)",
+        default=input["lectures_directory"],
+    )
+
+    input["example"] = prompt_list(
+        "Enter a git repository that contains an example project"
+    )
+
+    return input
+
+
 def fetch_latest(client, repository, **kwargs):
     _LOGGER.info(
         f'pulling latest version of the "{repository}" docker image, this may take a while...'
@@ -135,6 +238,24 @@ def create_volume(client, name):
     return client.volumes.create(f"{name}-{generate_random_string()}")
 
 
+def push_image(
+    client: docker.client, registry: string, image: docker.models.images.Image
+):
+
+    toPush = image.tags[0]
+    if registry is not None:
+        toPush = f"{registry}/{toPush}"
+        image.tag(toPush)
+
+    try:
+        # this can also return a generator to be used for a progress bar
+        client.images.push(toPush)
+    except:
+        _LOGGER.error(
+            "Failed to push the image to the registry, are you sure you have properly authenticated with the remote registry"
+        )
+
+
 def delete_volume(volume, **kwargs):
     _LOGGER.info("Deleteing setup volume...")
     volume.remove()
@@ -624,6 +745,9 @@ def run(opts, **kwargs):
     if opts["save_tar"] is not None:
         save_image(image, opts["save_tar"], "/output" if containerized else "./output")
 
+    if not opts["no_push"]:
+        push_image(client, opts["registry"], image)
+
 
 def main():
     args = docopt(__doc__)
@@ -677,6 +801,9 @@ def main():
     opts = _make_opts(args)
     _LOGGER.info(opts)
 
+    if opts["interactive"]:
+        opts = ask_interactive(opts)
+        _LOGGER.info(opts)
     run(opts)
 
 
